# Copyright (C) 2024  Loren Eteval <loren.eteval@proton.me>
#
# This file is part of Furious.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from __future__ import annotations

from Furious.Interface import *
from Furious.QtFramework import *
from Furious.QtFramework import gettext as _
from Furious.Library import *
from Furious.Utility import *
from Furious.Widget.UserServersQTableWidget import *
from Furious.Window.UserSubsWindow import *
from Furious.Window.LogViewerWindow import *
from Furious.Window.XrayAssetViewerWindow import *

from PySide6 import QtCore
from PySide6.QtGui import *
from PySide6.QtWidgets import *
from PySide6.QtNetwork import *

from typing import Union

import logging
import functools

__all__ = ['AppMainWindow']

logger = logging.getLogger(__name__)

registerAppSettings('ServerWidgetWindowSize')

needTrans = functools.partial(needTransFn, source=__name__)


def connectedHttpProxyEndpoint() -> Union[str, None]:
    try:
        if APP().isSystemTrayConnected():
            index = AS_UserActivatedItemIndex()

            if index >= 0:
                return AS_UserServers()[index].httpProxyEndpoint()
            else:
                # Should not reach here
                return None
        else:
            return None
    except Exception:
        # Any non-exit exceptions

        return None


def connectedRemark() -> str:
    try:
        if APP().isSystemTrayConnected():
            index = AS_UserActivatedItemIndex()

            if index >= 0:
                return f'{index + 1} - ' + AS_UserServers()[index].getExtras('remark')
            else:
                # Should not reach here
                return ''
        else:
            return ''
    except Exception:
        # Any non-exit exceptions

        return ''


class AppNetworkStateManager(NetworkStateManager):
    def __init__(self, parent=None):
        super().__init__(parent)

    def successCallback(self):
        parent = self.parent()

        if isinstance(parent, AppMainWindow):
            parent.setNetworkState(True)

    def errorCallback(self, errorString: str):
        parent = self.parent()

        if isinstance(parent, AppMainWindow):
            parent.setNetworkState(False, errorString=errorString)

    def startSingleTest(self):
        if not APP().isSystemTrayConnected():
            parent = self.parent()

            if isinstance(parent, AppMainWindow):
                parent.resetNetworkState()

            self.stopTest()

        httpProxyEndpoint = connectedHttpProxyEndpoint()

        if httpProxyEndpoint is None:
            parent = self.parent()

            if isinstance(parent, AppMainWindow):
                parent.resetNetworkState()
        else:
            self.configureHttpProxy(httpProxyEndpoint)

            super().startSingleTest()

    def disconnectedCallback(self):
        parent = self.parent()

        if isinstance(parent, AppMainWindow):
            parent.resetNetworkState()

        self.stopTest()


needTrans(
    'Server',
    'Add VMess Server...',
    'Add VLESS Server...',
    'Add Shadowsocks Server...',
    'Add Trojan Server...',
    'Add Hysteria1 Server...',
    'Add Hysteria2 Server...',
    'Subscription',
    'Update Subscription (Use Current Proxy)',
    'Update Subscription (Force Proxy)',
    'Update Subscription (No Proxy)',
    'Edit Subscription...',
    'Log',
    'Show Furious Log',
    'Show Core Log',
    'Show Tun2socks Log',
    'Tools',
    'Manage Xray-core Asset File...',
    'Check For Updates',
    'About',
    'Help',
)


class AppMainWindow(AppQMainWindow):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setWindowTitle(_(APPLICATION_NAME))

        self.updatesManager = UpdatesManager()
        self.networkStateManager = AppNetworkStateManager(parent=self)

        self.userServersQTableWidget = UserServersQTableWidget()
        self.userSubsWindow = UserSubsWindow(
            deleteUniqueCallback=lambda unique: self.userServersQTableWidget.deleteItemByIndex(
                list(
                    index
                    for index, server in enumerate(AS_UserServers())
                    if server.getExtras('subsId') == unique
                )
            )
        )
        self.xrayAssetViewerWindow = XrayAssetViewerWindow()

        self.mainTab = AppQTabWidget()
        self.mainTab.addTab(self.userServersQTableWidget, _('Server'))

        logActions = [
            AppQAction(
                _('Show Furious Log'),
                callback=lambda: APP().logViewerWindowApp_.showMaximized(),
                shortcut=QtCore.QKeyCombination(
                    QtCore.Qt.KeyboardModifier.ControlModifier
                    | QtCore.Qt.KeyboardModifier.ShiftModifier,
                    QtCore.Qt.Key.Key_F,
                ),
            ),
            AppQAction(
                _('Show Core Log'),
                callback=lambda: APP().logViewerWindowCore.showMaximized(),
                shortcut=QtCore.QKeyCombination(
                    QtCore.Qt.KeyboardModifier.ControlModifier
                    | QtCore.Qt.KeyboardModifier.ShiftModifier,
                    QtCore.Qt.Key.Key_C,
                ),
            ),
            AppQAction(
                _('Show Tun2socks Log'),
                callback=lambda: APP().logViewerWindowTun_.showMaximized(),
                shortcut=QtCore.QKeyCombination(
                    QtCore.Qt.KeyboardModifier.ControlModifier
                    | QtCore.Qt.KeyboardModifier.ShiftModifier,
                    QtCore.Qt.Key.Key_T,
                ),
            ),
        ]

        serverActions = [
            AppQAction(
                _('Add VMess Server...'),
                callback=lambda: self.userServersQTableWidget.addServerViaGui(
                    Protocol.VMess, _('Add VMess Server...')
                ),
            ),
            AppQAction(
                _('Add VLESS Server...'),
                callback=lambda: self.userServersQTableWidget.addServerViaGui(
                    Protocol.VLESS, _('Add VLESS Server...')
                ),
            ),
            AppQAction(
                _('Add Shadowsocks Server...'),
                callback=lambda: self.userServersQTableWidget.addServerViaGui(
                    Protocol.Shadowsocks, _('Add Shadowsocks Server...')
                ),
            ),
            AppQAction(
                _('Add Trojan Server...'),
                callback=lambda: self.userServersQTableWidget.addServerViaGui(
                    Protocol.Trojan, _('Add Trojan Server...')
                ),
            ),
            AppQSeperator(),
            AppQAction(
                _('Add Hysteria1 Server...'),
                callback=lambda: self.userServersQTableWidget.addServerViaGui(
                    Protocol.Hysteria1, _('Add Hysteria1 Server...')
                ),
            ),
            AppQAction(
                _('Add Hysteria2 Server...'),
                callback=lambda: self.userServersQTableWidget.addServerViaGui(
                    Protocol.Hysteria2, _('Add Hysteria2 Server...')
                ),
            ),
        ]

        subsActions = [
            AppQAction(
                _('Update Subscription (Use Current Proxy)'),
                callback=lambda: self.userServersQTableWidget.updateSubs(
                    connectedHttpProxyEndpoint()
                ),
            ),
            AppQAction(
                _('Update Subscription (Force Proxy)'),
                callback=lambda: self.userServersQTableWidget.updateSubs(
                    '127.0.0.1:10809'
                ),
            ),
            AppQAction(
                _('Update Subscription (No Proxy)'),
                callback=lambda: self.userServersQTableWidget.updateSubs(None),
            ),
            AppQSeperator(),
            AppQAction(
                _('Edit Subscription...'),
                icon=bootstrapIcon('star.svg'),
                callback=lambda: self.userSubsWindow.show(),
            ),
        ]

        toolsActions = [
            AppQAction(
                _('Manage Xray-core Asset File...'),
                callback=lambda: self.xrayAssetViewerWindow.show(),
            ),
        ]

        if hasattr(AppQAction, 'setMenu'):
            self.toolbar = AppQToolBar(
                AppQAction(
                    _('Log'),
                    icon=bootstrapIcon('pin-angle.svg'),
                    menu=AppQMenu(*logActions),
                    useActionGroup=False,
                    checkable=False,
                ),
                AppQSeperator(),
                AppQAction(
                    _('Server'),
                    icon=bootstrapIcon('database.svg'),
                    menu=AppQMenu(*serverActions),
                    useActionGroup=False,
                    checkable=False,
                ),
                AppQSeperator(),
                AppQAction(
                    _('Subscription'),
                    icon=bootstrapIcon('collection.svg'),
                    menu=AppQMenu(*subsActions),
                    useActionGroup=False,
                    checkable=False,
                ),
                AppQSeperator(),
                AppQAction(
                    _('Tools'),
                    icon=bootstrapIcon('tools.svg'),
                    menu=AppQMenu(*toolsActions),
                    useActionGroup=False,
                    checkable=False,
                ),
                AppQSeperator(),
                AppQAction(
                    _('Check For Updates'),
                    icon=bootstrapIcon('download.svg'),
                    checkable=False,
                    callback=lambda: self.checkForUpdates(),
                ),
                AppQSeperator(),
                AppQAction(
                    _('About'),
                    icon=bootstrapIcon('info-circle.svg'),
                    checkable=False,
                    callback=lambda: self.openAboutPage(),
                ),
            )
            self.toolbar.setIconSize(QtCore.QSize(64, 32))
            self.toolbar.setToolButtonStyle(
                QtCore.Qt.ToolButtonStyle.ToolButtonTextUnderIcon
            )
            self.addToolBar(self.toolbar)
        else:
            # Menu actions
            logMenu = {
                'name': 'Log',
                'actions': [*logActions],
            }

            serverMenu = {
                'name': 'Server',
                'actions': [*serverActions],
            }

            subsMenu = {
                'name': 'Subscription',
                'actions': [*subsActions],
            }

            toolsMenu = {
                'name': 'Tools',
                'actions': [*toolsActions],
            }

            helpMenu = {
                'name': 'Help',
                'actions': [
                    AppQAction(
                        _('Check For Updates'),
                        icon=bootstrapIcon('download.svg'),
                        checkable=False,
                        callback=lambda: self.checkForUpdates(),
                    ),
                    AppQSeperator(),
                    AppQAction(
                        _('About'),
                        icon=bootstrapIcon('info-circle.svg'),
                        checkable=False,
                        callback=lambda: self.openAboutPage(),
                    ),
                ],
            }

            # Menus
            for menuDict in (logMenu, serverMenu, subsMenu, toolsMenu, helpMenu):
                menuName = menuDict['name']
                menuObjName = f'_{menuName}Menu'
                menu = AppQMenu(
                    *menuDict['actions'], title=_(menuName), parent=self.menuBar()
                )

                # Set reference
                setattr(self, menuObjName, menu)

                self.menuBar().addMenu(menu)

        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)

        # TODO: Custom status tip
        # self.setStatusBar(QStatusBar(self))

        self.networkState = AppQLabel(translatable=False)

        self.statusBar().addPermanentWidget(self.networkState)

        self.networkStateManager.startTest()

        self._widget = QWidget()
        self._layout = QVBoxLayout(self._widget)
        self._layout.addWidget(self.mainTab)

        self.setCentralWidget(self._widget)

    def appendNewItemByFactory(self, factory: ConfigurationFactory):
        self.userServersQTableWidget.appendNewItemByFactory(factory)

    def flushRow(self, row: int, item: ConfigurationFactory):
        self.userServersQTableWidget.flushRow(row, item)

    def showTabAndSpaces(self):
        self.userServersQTableWidget.showTabAndSpaces()

    def hideTabAndSpaces(self):
        self.userServersQTableWidget.hideTabAndSpaces()

    def checkForUpdates(self):
        self.updatesManager.configureHttpProxy(connectedHttpProxyEndpoint())
        self.updatesManager.checkForUpdates()

    def resetNetworkState(self):
        self.networkState.setText('')

    def setNetworkState(self, success: bool, **kwargs):
        remark = connectedRemark()

        if success:
            if remark:
                self.networkState.setText(f'{remark} {UNICODE_LARGE_GREEN_CIRCLE}')
            else:
                self.resetNetworkState()
        else:
            errorString = kwargs.pop('errorString', '')

            if remark:
                self.networkState.setText(
                    f'{remark} - {errorString} {UNICODE_LARGE_RED_CIRCLE}'
                )
            else:
                self.resetNetworkState()

    @staticmethod
    def openAboutPage():
        if QDesktopServices.openUrl(QtCore.QUrl(APPLICATION_ABOUT_PAGE)):
            logger.info('open about page success')
        else:
            logger.error('open about page failed')

    def setWidthAndHeight(self):
        try:
            windowSize = AppSettings.get('ServerWidgetWindowSize').split(',')

            self.setGeometry(100, 100, *list(int(size) for size in windowSize))
        except Exception:
            # Any non-exit exceptions

            self.setGeometry(100, 100, 1800, 960)

    def cleanup(self):
        AppSettings.set(
            'ServerWidgetWindowSize',
            f'{self.geometry().width()},{self.geometry().height()}',
        )



# Copyright (C) 2024  Loren Eteval <loren.eteval@proton.me>
#
# This file is part of Furious.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from __future__ import annotations

from Furious.Interface import *
from Furious.QtFramework import *
from Furious.QtFramework import gettext as _
from Furious.Utility import *

from PySide6 import QtCore
from PySide6.QtGui import *
from PySide6.QtWidgets import *

import logging
import functools

__all__ = ['GuiHysteria2']

logger = logging.getLogger(__name__)

needTrans = functools.partial(needTransFn, source=__name__)


class GuiHy2ItemBasicServer(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        oldServer = config.get('server', '')
        newServer = self.text()

        if isinstance(oldServer, str):
            if newServer != oldServer:
                config['server'] = newServer

                return True
            else:
                return False
        else:
            config['server'] = newServer

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            self.setText(config.get('server', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiHy2ItemBasicAuth(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        oldAuth = config.get('auth')
        newAuth = self.text()

        if newAuth == '':
            if oldAuth is None:
                return False
            else:
                config.pop('auth', None)

                return True
        else:
            if isinstance(oldAuth, str):
                if newAuth != oldAuth:
                    config['auth'] = newAuth

                    return True
                else:
                    return False
            else:
                config['auth'] = newAuth

                return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            self.setText(config.get('auth', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiHy2ItemObfsType(GuiEditorItemTextComboBox):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # TODO: Only 'salamander' is currently supported by hy2
        self.addItems(['', 'salamander'])

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        try:
            oldObfsType = config['obfs']['type']
        except Exception:
            # Any non-exit exceptions

            oldObfsType = ''

        newObfsType = self.text()

        if newObfsType == '':
            if config.get('obfs') is None:
                return False
            else:
                config.pop('obfs', None)

                return True
        else:
            if not isinstance(oldObfsType, str) or newObfsType != oldObfsType:
                if not isinstance(config.get('obfs'), dict):
                    config['obfs'] = {}

                try:
                    config['obfs']['type'] = newObfsType
                except Exception:
                    # Any non-exit exceptions

                    pass

                return True
            else:
                return False

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            obfsType = config['obfs']['type']
        except Exception:
            # Any non-exit exceptions

            obfsType = ''

        # TODO: Only 'salamander' is currently supported by hy2
        if isinstance(obfsType, str) and obfsType != 'salamander':
            self.setText('')
        else:
            self.setText('salamander')


class GuiHy2ItemObfsPassword(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        newObfsPassword = self.text()

        try:
            obfsType = config['obfs']['type']
        except Exception:
            # Any non-exit exceptions

            obfsType = ''

        # TODO: Only 'salamander' is currently supported by hy2
        if not isinstance(obfsType, str) or obfsType != 'salamander':
            if config.get('obfs') is None:
                return False
            else:
                # Remove obfs field
                config.pop('obfs', None)

                return True

        modified = False

        if not isinstance(config.get('obfs'), dict):
            config['obfs'] = {}

            modified = True

        if not isinstance(config['obfs'].get(obfsType), dict):
            config['obfs'][obfsType] = {}

            modified = True

        try:
            oldObfsPassword = config['obfs'][obfsType]['password']
        except Exception:
            # Any non-exit exceptions

            oldObfsPassword = ''

        if not isinstance(oldObfsPassword, str) or newObfsPassword != oldObfsPassword:
            config['obfs'][obfsType]['password'] = newObfsPassword

            return True
        else:
            return modified

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            obfsType = config['obfs']['type']
        except Exception:
            # Any non-exit exceptions

            obfsType = ''

        # TODO: Only 'salamander' is currently supported by hy2
        if not isinstance(obfsType, str) or obfsType != 'salamander':
            self.setText('')

            return

        try:
            obfsPassword = config['obfs'][obfsType]['password']
        except Exception:
            # Any non-exit exceptions

            obfsPassword = ''

        self.setText(obfsPassword)


class GuiHy2ItemTLSTextInput(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        key = kwargs.pop('key', '')

        super().__init__(*args, **kwargs)

        self.key = key

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        newValue = self.text()

        if newValue == '':
            tls = config.get('tls')

            if isinstance(tls, dict):
                if len(tls) > 0:
                    if tls.get(self.key) is not None:
                        oldValue = tls[self.key]

                        tls.pop(self.key, None)

                        if not isinstance(oldValue, str) or oldValue != '':
                            modified = True
                        else:
                            # Modified silently
                            modified = False
                    else:
                        modified = False

                    # TODO: Future extension
                    anyValid = any(
                        tls.get(key, '')
                        for key in ['sni', 'insecure', 'pinSHA256', 'ca']
                    )

                    if not anyValid:
                        config.pop('tls', None)

                        return True
                    else:
                        return modified
                else:
                    config.pop('tls', None)

                    # Modified silently
                    return False
            else:
                config.pop('tls', None)

                # Modified silently
                return False
        else:
            if not isinstance(config.get('tls'), dict):
                config['tls'] = {}

            oldValue = config['tls'].get(self.key, '')

            if not isinstance(oldValue, str) or newValue != oldValue:
                config['tls'][self.key] = newValue

                return True
            else:
                return False

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            value = config['tls'][self.key]
        except Exception:
            # Any non-exit exceptions

            value = ''

        self.setText(value)


class GuiHy2ItemTLSInsecure(GuiEditorItemTextCheckBox):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        try:
            oldChecked = config['tls']['insecure']
        except Exception:
            # Any non-exit exceptions

            oldChecked = False

        newChecked = self.isChecked()

        if newChecked:
            if not isinstance(config.get('tls'), dict):
                config['tls'] = {}

            if oldChecked is not True:
                config['tls']['insecure'] = True

                return True
            else:
                return False
        else:
            if not isinstance(config.get('tls'), dict):
                config['tls'] = {}

            if oldChecked is not False:
                config['tls']['insecure'] = False

                return True
            else:
                return False

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            checked = config['tls']['insecure']
        except Exception:
            # Any non-exit exceptions

            checked = False

        if not isinstance(checked, bool):
            checked = False

        self.setChecked(checked)


needTrans(
    'Basic Configuration',
    'Remark',
    'Server',
)


class GuiHy2GroupBoxBasic(GuiEditorWidgetQGroupBox):
    def __init__(self, **kwargs):
        super().__init__(_('Basic Configuration'), **kwargs)

    def containerSequence(self):
        return [
            GuiEditorItemBasicRemark(title=_('Remark')),
            GuiHy2ItemBasicServer(title=_('Server')),
            GuiHy2ItemBasicAuth(title='auth', translatable=False),
        ]


needTrans('Proxy')


class GuiHy2GroupBoxProxy(GuiEditorWidgetQGroupBox):
    def __init__(self, **kwargs):
        super().__init__(_('Proxy'), **kwargs)

    def containerSequence(self):
        return [
            GuiEditorItemProxyHttp(title='http', translatable=False),
            GuiEditorItemProxySocks(title='socks', translatable=False),
        ]


class GuiHy2GroupBoxObfs(GuiEditorWidgetQGroupBox):
    def __init__(self, **kwargs):
        super().__init__('obfs', **kwargs)

        self.translatable = False

    def containerSequence(self):
        return [
            GuiHy2ItemObfsType(title='obfs-type', translatable=False),
            GuiHy2ItemObfsPassword(title='obfs-password', translatable=False),
        ]


class GuiHy2GroupBoxTLS(GuiEditorWidgetQGroupBox):
    def __init__(self, **kwargs):
        super().__init__('TLS', **kwargs)

        self.translatable = False

    def containerSequence(self):
        return [
            GuiHy2ItemTLSTextInput(title='sni', key='sni', translatable=False),
            GuiHy2ItemTLSTextInput(
                title='pinSHA256', key='pinSHA256', translatable=False
            ),
            GuiHy2ItemTLSTextInput(title='ca', key='ca', translatable=False),
            GuiHy2ItemTLSInsecure(title='insecure', translatable=False),
        ]


needTrans('Project Website')


class GuiHy2ProjectWebsiteURL(AppQLabel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setWebsiteURL()
        self.linkActivated.connect(self.handleLinkActivated)

    def setWebsiteURL(self):
        self.setText(
            '<html><head/><body><p>'
            '<a href=\"https://v2.hysteria.network/\">'
            '<span style=\" text-decoration: underline; color:#007ad6;\">'
            + _('Project Website')
            + '</span></a></p></body></html>'
        )

    @staticmethod
    def handleLinkActivated(link: str):
        if QDesktopServices.openUrl(QtCore.QUrl(link)):
            logger.info(f'open link \'{link}\' success')
        else:
            logger.error(f'open link \'{link}\' failed')

    def retranslate(self):
        self.setWebsiteURL()


needTrans('Other')


class GuiHy2GroupBoxOther(GuiEditorItemFactory, AppQGroupBox):
    def __init__(self, **kwargs):
        super().__init__(_('Other'), **kwargs)

        self._website = GuiHy2ProjectWebsiteURL()

        layout = QFormLayout()
        layout.addRow(self._website)

        self.setLayout(layout)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        return False

    def factoryToInput(self, config: ConfigurationFactory):
        pass


class GuiHysteria2(GuiEditorWidgetQDialog):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setTabText(Protocol.Hysteria2)

    def groupBoxSequence(self):
        return [
            GuiHy2GroupBoxBasic(),
            GuiHy2GroupBoxProxy(),
            GuiHy2GroupBoxObfs(),
            GuiHy2GroupBoxTLS(),
            # GuiHy2GroupBoxOther(),
        ]



TRANSLATION = {
    "Delete": {
        "source": [
            "Furious.QtFramework.QtWidgets",
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Widget.UserSubsQTableWidget",
            "Furious.Window.UserSubsWindow",
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Удалить",
        "ZH": "删除",
        "isReviewed": "True"
    },
    "Delete these items?": {
        "source": [
            "Furious.QtFramework.QtWidgets"
        ],
        "RU": "Удалить эти элементы",
        "ZH": "删除这些项目？",
        "isReviewed": "True"
    },
    "Delete this item?": {
        "source": [
            "Furious.QtFramework.QtWidgets"
        ],
        "RU": "Удалить этот элемент",
        "ZH": "删除此项目？",
        "isReviewed": "True"
    },
    "New changes will take effect next time": {
        "source": [
            "Furious.QtFramework.QtWidgets"
        ],
        "RU": "Новые изменения вступят в силу в следующий раз",
        "ZH": "新的更改将于下次生效",
        "isReviewed": "True"
    },
    "New version available": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Доступна новая версия",
        "ZH": "新版本可用",
        "isReviewed": "True"
    },
    "Check for updates failed": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Не удалось проверить наличие обновлений",
        "ZH": "检查更新失败",
        "isReviewed": "True"
    },
    "Go to download page?": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Перейти на страницу загрузки?",
        "ZH": "前往下载页面？",
        "isReviewed": "True"
    },
    "Furious is already the latest version": {
        "source": [
            "Furious.QtFramework.UpdatesManager"
        ],
        "RU": "Furious - это уже последняя версия",
        "ZH": "Furious已是最新版本",
        "isReviewed": "True"
    },
    "Invalid data": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Неверные данные",
        "ZH": "无效的数据",
        "isReviewed": "True"
    },
    "Invalid data. The content of the clipboard is:": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Неверные данные. Содержимое буфера обмена является:",
        "ZH": "无效的数据。剪贴板内容为",
        "isReviewed": "True"
    },
    "Import": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Импорт",
        "ZH": "导入",
        "isReviewed": "True"
    },
    "Import share link success": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Успешный импорт общей ссылки",
        "ZH": "导入分享链接成功",
        "isReviewed": "True"
    },
    "Imported to row": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Импорт в строку",
        "ZH": "导入至行",
        "isReviewed": "True"
    },
    "Import success": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Успешный импорт",
        "ZH": "导入成功",
        "isReviewed": "True"
    },
    "Import From File...": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Импорт из файла...",
        "ZH": "从文件导入...",
        "isReviewed": "True"
    },
    "Import File": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Импорт файла",
        "ZH": "导入文件",
        "isReviewed": "True"
    },
    "Text files (*.json);;All files (*)": {
        "source": [
            "Furious.TrayActions.Import",
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Текст файлов (*.json);;Остальные файлы (*)",
        "ZH": "文本文件 (*.json);;所有文件 (*)",
        "isReviewed": "True"
    },
    "Error opening file": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Ошибка открытия файла",
        "ZH": "打开文件出错",
        "isReviewed": "True"
    },
    "Invalid configuration file": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Недопустимый конфигурационный файл",
        "ZH": "无效的配置文件",
        "isReviewed": "True"
    },
    "Import Share Link From Clipboard": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Импортировать ссылку с буфера обмена",
        "ZH": "从剪贴板导入分享链接",
        "isReviewed": "True"
    },
    "Import JSON Configuration From Clipboard": {
        "source": [
            "Furious.TrayActions.Import"
        ],
        "RU": "Импортировать Json конфигурацию с буфера обмена",
        "ZH": "从剪贴板导入JSON配置",
        "isReviewed": "True"
    },
    "Set Indent": {
        "source": [
            "Furious.Widget.IndentSpinBox"
        ],
        "RU": "Установить отступ",
        "ZH": "设置缩进",
        "isReviewed": "True"
    },
    "Indent:": {
        "source": [
            "Furious.Widget.IndentSpinBox"
        ],
        "RU": "Отступ",
        "ZH": "缩进：",
        "isReviewed": "True"
    },
    "Cancel": {
        "source": [
            "Furious.QtFramework.GuiEditorXXX",
            "Furious.Widget.IndentSpinBox",
            "Furious.Window.TextEditorWindow",
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Отменить",
        "ZH": "取消",
        "isReviewed": "True"
    },
    "Save Changes": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Сохранить изменения",
        "ZH": "保存更改",
        "isReviewed": "True"
    },
    "The content has been modified. Save changes?": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Содержимое было изменено. Сохранить изменения?",
        "ZH": "内容已修改。保存更改？",
        "isReviewed": "True"
    },
    "Save": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Сохранить",
        "ZH": "保存",
        "isReviewed": "True"
    },
    "Discard": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Отбросить",
        "ZH": "丢弃",
        "isReviewed": "True"
    },
    "Please check if the configuration is in valid JSON format": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "В конфигурации недопустимый формат JSON, проверьте",
        "ZH": "请检查配置是否为有效的JSON格式",
        "isReviewed": "True"
    },
    "Save As...": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Сохранить как...",
        "ZH": "另存为...",
        "isReviewed": "True"
    },
    "Exit": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow",
            "Furious.Window.XrayAssetViewerWindow",
            "Furious.TrayActions.Exit"
        ],
        "RU": "Выход",
        "ZH": "退出",
        "isReviewed": "True"
    },
    "File": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow",
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Файл",
        "ZH": "文件",
        "isReviewed": "True"
    },
    "Undo": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Отменить",
        "ZH": "撤销",
        "isReviewed": "True"
    },
    "Redo": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Переделать",
        "ZH": "重做",
        "isReviewed": "True"
    },
    "Cut": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Вырезать",
        "ZH": "剪切",
        "isReviewed": "True"
    },
    "Copy": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Копировать",
        "ZH": "复制",
        "isReviewed": "True"
    },
    "Paste": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Вставить",
        "ZH": "粘贴",
        "isReviewed": "True"
    },
    "Select All": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Выбрать все",
        "ZH": "全选",
        "isReviewed": "True"
    },
    "Indent...": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Отступ",
        "ZH": "缩进...",
        "isReviewed": "True"
    },
    "Edit": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Редактировать",
        "ZH": "编辑",
        "isReviewed": "True"
    },
    "Zoom In": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Увеличить маштаб",
        "ZH": "变大",
        "isReviewed": "True"
    },
    "Zoom Out": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Уменьшить маштаб",
        "ZH": "变小",
        "isReviewed": "True"
    },
    "View": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Вид",
        "ZH": "显示",
        "isReviewed": "True"
    },
    "Error saving configuration": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Конфигурация с сохранением ошибок",
        "ZH": "保存配置出错",
        "isReviewed": "True"
    },
    "Save File": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Сохранить файл",
        "ZH": "保存文件",
        "isReviewed": "True"
    },
    "Error Saving File": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Ошибка сохранения файла",
        "ZH": "保存文件出错",
        "isReviewed": "True"
    },
    "Invalid server configuration": {
        "source": [
            "Furious.Window.TextEditorWindow",
            "Furious.TrayActions.Connect"
        ],
        "RU": "Неверная конфигурация сервера",
        "ZH": "无效的服务器配置",
        "isReviewed": "True"
    },
    "Error setting indent": {
        "source": [
            "Furious.Window.TextEditorWindow"
        ],
        "RU": "Ошибка при установке отступов внутрь и наружу",
        "ZH": "设置缩进出错",
        "isReviewed": "True"
    },
    "Remark": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Widget.UserSubsQTableWidget"
        ],
        "RU": "Прозвище",
        "ZH": "别名",
        "isReviewed": "True"
    },
    "Protocol": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Протокол",
        "ZH": "协议",
        "isReviewed": "True"
    },
    "Address": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Адрес",
        "ZH": "地址",
        "isReviewed": "True"
    },
    "Port": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Порт",
        "ZH": "端口",
        "isReviewed": "True"
    },
    "Transport": {
        "source": [
            "Furious.Widget.GuiVTransport",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Способ передачи",
        "ZH": "传输方式",
        "isReviewed": "True"
    },
    "TLS": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "TLS",
        "ZH": "TLS",
        "isReviewed": "True"
    },
    "Subscription": {
        "source": [
            "Furious.Widget.UserServersQTableWidget",
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Подписка",
        "ZH": "订阅",
        "isReviewed": "True"
    },
    "Latency": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Задержка",
        "ZH": "延迟",
        "isReviewed": "True"
    },
    "Speed": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Скорость",
        "ZH": "速度",
        "isReviewed": "True"
    },
    "Edit Configuration...": {
        "source": [
            "Furious.TrayActions.EditConfiguration"
        ],
        "RU": "Отредактируйте конфигурацию...",
        "ZH": "编辑配置...",
        "isReviewed": "True"
    },
    "Move Up": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Двигать вверх",
        "ZH": "上移",
        "isReviewed": "True"
    },
    "Move Down": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Двигать в низ",
        "ZH": "下移",
        "isReviewed": "True"
    },
    "Duplicate": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Дубликат",
        "ZH": "克隆",
        "isReviewed": "True"
    },
    "Scroll To Activated Server": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Перейдите к активированному серверу",
        "ZH": "滚动到激活的服务器",
        "isReviewed": "True"
    },
    "Test Ping Latency": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Задержка тестового пинга",
        "ZH": "测试Ping延迟",
        "isReviewed": "True"
    },
    "Test Download Speed": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Проверьте скорость загрузки",
        "ZH": "测试下行速度",
        "isReviewed": "True"
    },
    "Clear Test Results": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Четкие результаты тестирования",
        "ZH": "清除测试结果",
        "isReviewed": "True"
    },
    "New Empty Configuration": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Создайте новую пустую конфигурацию",
        "ZH": "新建空白配置",
        "isReviewed": "True"
    },
    "Export Share Link To Clipboard": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Копировать ссылку в буфер обмена",
        "ZH": "导出分享链接至剪贴板",
        "isReviewed": "True"
    },
    "Export As QR Code": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Экспорт в виде QR-кода",
        "ZH": "导出为二维码",
        "isReviewed": "True"
    },
    "Export JSON Configuration To Clipboard": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Копировать конфигурацию JSON в буфер обмена",
        "ZH": "导出JSON配置至剪贴板",
        "isReviewed": "True"
    },
    "Connecting": {
        "source": [
            "Furious.Widget.UserServersQTableWidget",
            "Furious.TrayActions.Connect"
        ],
        "RU": "Соединение",
        "ZH": "正在连接",
        "isReviewed": "True"
    },
    "Connecting. Please wait": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Соединение. Пожалуста подождите",
        "ZH": "正在连接。请稍候",
        "isReviewed": "True"
    },
    "Untitled": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Без названия",
        "ZH": "未命名",
        "isReviewed": "True"
    },
    "Add Subscription": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Добавить подписку",
        "ZH": "添加订阅",
        "isReviewed": "True"
    },
    "Enter subscription remark:": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Введите примечание о подписке:",
        "ZH": "输入订阅名称：",
        "isReviewed": "True"
    },
    "Enter subscription URL:": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Введите URL-адрес подписки:",
        "ZH": "输入订阅URL：",
        "isReviewed": "True"
    },
    "Edit Subscription": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Редактировать подписку",
        "ZH": "编辑订阅",
        "isReviewed": "True"
    },
    "Subscription List": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Список подписки",
        "ZH": "订阅列表",
        "isReviewed": "True"
    },
    "Add": {
        "source": [
            "Furious.Window.UserSubsWindow"
        ],
        "RU": "Добавить",
        "ZH": "添加",
        "isReviewed": "True"
    },
    "Unable to save log": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Не удалось сохранить журнал",
        "ZH": "无法保存日志",
        "isReviewed": "True"
    },
    "Text files (*.txt);;All files (*)": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Текстовые файлы (*.txt);;Все файлы (*)",
        "ZH": "文本文件 (*.txt);;所有文件 (*)",
        "isReviewed": "True"
    },
    "Error saving log": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Журнал сохранения ошибок",
        "ZH": "保存日志出错",
        "isReviewed": "True"
    },
    "Log Viewer": {
        "source": [
            "Furious.Window.LogViewerWindow"
        ],
        "RU": "Средство просмотра журналов",
        "ZH": "日志查看器",
        "isReviewed": "True"
    },
    "Asset file already exists. Overwrite?": {
        "source": [
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Файл ресурсов уже существует. Перезаписать?",
        "ZH": "资源文件已存在。是否覆盖？",
        "isReviewed": "True"
    },
    "Error import asset file": {
        "source": [
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Ошибка при импорте файла актива",
        "ZH": "导入资源文件出错",
        "isReviewed": "True"
    },
    "Import asset file success": {
        "source": [
            "Furious.Widget.XrayAssetViewerQListWidget"
        ],
        "RU": "Успешно импортирован файл ресурсов",
        "ZH": "导入资源文件成功",
        "isReviewed": "True"
    },
    "Xray-core Asset File": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Xray-core Asset File",
        "ZH": "Xray-core资源文件",
        "isReviewed": "True"
    },
    "Refresh": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Обновить",
        "ZH": "刷新",
        "isReviewed": "True"
    },
    "Open Asset Directory": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Откройте каталог ресурсов",
        "ZH": "打开资源目录",
        "isReviewed": "True"
    },
    "All files (*)": {
        "source": [
            "Furious.Window.XrayAssetViewerWindow"
        ],
        "RU": "Все файлы (*)",
        "ZH": "所有文件 (*)",
        "isReviewed": "True"
    },
    "Server": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Сервер",
        "ZH": "服务器",
        "isReviewed": "True"
    },
    "Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Журнал событий",
        "ZH": "日志",
        "isReviewed": "True"
    },
    "Show Furious Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Показать журнал Furious",
        "ZH": "显示Furious日志",
        "isReviewed": "True"
    },
    "Show Core Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Показать журнал ядра",
        "ZH": "显示内核日志",
        "isReviewed": "True"
    },
    "Show Tun2socks Log": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Показать журнал Tun2socks",
        "ZH": "显示Tun2socks日志",
        "isReviewed": "True"
    },
    "Update Subscription (Use Current Proxy)": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Обновление подписки (используйте текущий прокси-сервер)",
        "ZH": "更新订阅（使用当前代理）",
        "isReviewed": "True"
    },
    "Update Subscription (Force Proxy)": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Обновление подписки (обязательный прокси-сервер)",
        "ZH": "更新订阅（强制代理）",
        "isReviewed": "True"
    },
    "Update Subscription (No Proxy)": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Обновление подписки (не через прокси)",
        "ZH": "更新订阅（无代理）",
        "isReviewed": "True"
    },
    "Edit Subscription...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Отредактируйте подписку...",
        "ZH": "编辑订阅...",
        "isReviewed": "True"
    },
    "Tools": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Иструменты",
        "ZH": "工具",
        "isReviewed": "True"
    },
    "Manage Xray-core Asset File...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Управляйте файлом обьекта Xray-core...",
        "ZH": "管理Xray-core资源文件...",
        "isReviewed": "True"
    },
    "Check For Updates": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Проверка обновлений",
        "ZH": "检查更新",
        "isReviewed": "True"
    },
    "About": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "О нас",
        "ZH": "关于",
        "isReviewed": "True"
    },
    "Already started": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Активированный",
        "ZH": "已启动",
        "isReviewed": "True"
    },
    "Furious Log": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Furious журнал",
        "ZH": "Furious日志",
        "isReviewed": "True"
    },
    "Core Log": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Лог ядра",
        "ZH": "内核日志",
        "isReviewed": "True"
    },
    "Tun2socks Log": {
        "source": [
            "Furious.Widget.Application"
        ],
        "RU": "Лог Tun2socks",
        "ZH": "Tun2socks日志",
        "isReviewed": "True"
    },
    "Connect": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Соединение",
        "ZH": "连接",
        "isReviewed": "True"
    },
    "Disconnect": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Рассоединение",
        "ZH": "断开连接",
        "isReviewed": "True"
    },
    "Unable to connect": {
        "source": [
            "Furious.Core.CoreManager",
            "Furious.TrayActions.Connect"
        ],
        "RU": "Не удается подключиться",
        "ZH": "无法连接",
        "isReviewed": "True"
    },
    "Server configuration empty. Please configure your server first": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Конфигурация сервера пуста. Пожалуйста, сначала настройте свой сервер",
        "ZH": "服务器配置为空。请先配置服务器",
        "isReviewed": "True"
    },
    "Connected": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Подключенный",
        "ZH": "已连接",
        "isReviewed": "True"
    },
    "Unknown error": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Неизвестная ошибка",
        "ZH": "未知错误",
        "isReviewed": "True"
    },
    "Failed to start core": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Сбой при запуске ядра",
        "ZH": "内核启动失败",
        "isReviewed": "True"
    },
    "Connection to server has been lost": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Соединение с сервером было потеряно",
        "ZH": "与服务器的连接已丢失",
        "isReviewed": "True"
    },
    "Core terminated unexpectedly": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Неожиданное завершение работы ядра",
        "ZH": "内核意外终止",
        "isReviewed": "True"
    },
    "Disconnected": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Разъединенный",
        "ZH": "已断开",
        "isReviewed": "True"
    },
    "Language": {
        "source": [
            "Furious.TrayActions.Language"
        ],
        "RU": "Языки",
        "ZH": "语言",
        "isReviewed": "True"
    },
    "Routing": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "Маршрутизация",
        "ZH": "路由",
        "isReviewed": "True"
    },
    "VPN Mode": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "VPN режим",
        "ZH": "VPN模式",
        "isReviewed": "True"
    },
    "VPN Mode Disabled (Administrator)": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "VPN режим отключен (Administrator)",
        "ZH": "VPN模式已禁用（管理员）",
        "isReviewed": "True"
    },
    "VPN Mode Disabled (root)": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "VPN режим отключен  (root)",
        "ZH": "VPN模式已禁用（root）",
        "isReviewed": "True"
    },
    "Settings": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Настройки",
        "ZH": "设置",
        "isReviewed": "True"
    },
    "Dark Mode": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Темный режим",
        "ZH": "深色模式",
        "isReviewed": "True"
    },
    "Startup On Boot": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Автозапуск",
        "ZH": "开机启动",
        "isReviewed": "True"
    },
    "Show Progress Bar When Connecting": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Отображение индикатора выполнения при подключении",
        "ZH": "连接时显示进度条",
        "isReviewed": "True"
    },
    "Show Tab And Spaces In Editor": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Показывать табуляцию и пробелы в редакторе",
        "ZH": "在编辑器中显示制表符和空格",
        "isReviewed": "True"
    },
    "System Proxy": {
        "source": [
            "Furious.TrayActions.SystemProxy"
        ],
        "RU": "Системный прокси",
        "ZH": "系统代理",
        "isReviewed": "True"
    },
    "Automatically Configure System Proxy": {
        "source": [
            "Furious.TrayActions.SystemProxy"
        ],
        "RU": "Автоматическая настройка системного прокси-сервера",
        "ZH": "自动配置系统代理",
        "isReviewed": "True"
    },
    "Do Not Change System Proxy": {
        "source": [
            "Furious.TrayActions.SystemProxy"
        ],
        "RU": "Не меняйте системный прокси-сервер",
        "ZH": "不修改系统代理",
        "isReviewed": "True"
    },
    "Furious is not able to run on this operating system": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Furious не может работать в этой операционной системе",
        "ZH": "Furious无法在此操作系统上运行",
        "isReviewed": "True"
    },
    "Operating system information": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Информация об операционной системе",
        "ZH": "操作系统信息",
        "isReviewed": "True"
    },
    "Furious encountered an internal error and needs to be stopped": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "В Furious произошла внутренняя ошибка, и ее необходимо завершить",
        "ZH": "Furious遇到内部错误，需要终止",
        "isReviewed": "True"
    },
    "Furious stopped unexpectedly due to an unknown exception": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Программа Furious неожиданно завершилась из-за неизвестного исключения",
        "ZH": "由于一个未知异常，Furious意外终止",
        "isReviewed": "True"
    },
    "Crash log has been saved to": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Журнал сбоев был сохранен в",
        "ZH": "崩溃日志已保存到",
        "isReviewed": "True"
    },
    "Open crash log": {
        "source": [
            "Furious.__main__"
        ],
        "RU": "Открыть журнал ошибок",
        "ZH": "打开崩溃日志",
        "isReviewed": "True"
    },
    "Bypass Mainland China": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "В обход Китая",
        "ZH": "绕过中国大陆",
        "isReviewed": "True"
    },
    "Global": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "Весь трафик",
        "ZH": "全球",
        "isReviewed": "True"
    },
    "Custom": {
        "source": [
            "Furious.TrayActions.Routing"
        ],
        "RU": "Свои настройки",
        "ZH": "自定义",
        "isReviewed": "True"
    },
    "Furious cannot find any valid http proxy endpoint in the configuration": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Я не могу найти ни одной допустимой конечной точки http-прокси в конфигурации",
        "ZH": "Furious在配置中无法找到任何有效的http代理端点",
        "isReviewed": "True"
    },
    "Please complete your server configuration": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Пожалуйста, завершите настройку вашего сервера",
        "ZH": "请完善服务器配置",
        "isReviewed": "True"
    },
    "Help": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Помощь",
        "ZH": "帮助",
        "isReviewed": "True"
    },
    "Test Tcping Latency": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Задержка Tcping теста",
        "ZH": "测试Tcping延迟",
        "isReviewed": "True"
    },
    "Basic Configuration": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess"
        ],
        "RU": "Базовая конфигурация",
        "ZH": "基本配置",
        "isReviewed": "True"
    },
    "Proxy": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2",
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan",
            "Furious.Widget.GuiVLESS",
            "Furious.Widget.GuiVMess"
        ],
        "RU": "Прокси",
        "ZH": "代理",
        "isReviewed": "True"
    },
    "Project Website": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2"
        ],
        "RU": "Веб-сайт проекта",
        "ZH": "项目网站",
        "isReviewed": "True"
    },
    "Other": {
        "source": [
            "Furious.Widget.GuiHysteria1",
            "Furious.Widget.GuiHysteria2"
        ],
        "RU": "Прочеи",
        "ZH": "其它",
        "isReviewed": "True"
    },
    "Encryption": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiVLESS"
        ],
        "RU": "Шифрование",
        "ZH": "加密",
        "isReviewed": "True"
    },
    "Password": {
        "source": [
            "Furious.Widget.GuiShadowsocks",
            "Furious.Widget.GuiTrojan"
        ],
        "RU": "Пароль",
        "ZH": "密码",
        "isReviewed": "True"
    },
    "Flow": {
        "source": [
            "Furious.Widget.GuiVLESS"
        ],
        "RU": "Поток",
        "ZH": "流控",
        "isReviewed": "True"
    },
    "Use Monochrome Tray Icon": {
        "source": [
            "Furious.TrayActions.Settings"
        ],
        "RU": "Использование монохромного значока в трее",
        "ZH": "使用单色托盘图标",
        "isReviewed": "True"
    },
    "Customize JSON Configuration...": {
        "source": [
            "Furious.Widget.UserServersQTableWidget"
        ],
        "RU": "Настройте конфигурацию JSON...",
        "ZH": "自定义JSON配置...",
        "isReviewed": "True"
    },
    "Add VMess Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавить сервер VMees...",
        "ZH": "添加VMess服务器...",
        "isReviewed": "True"
    },
    "Add VLESS Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавить сервер VLESS...",
        "ZH": "添加VLESS服务器...",
        "isReviewed": "True"
    },
    "Add Shadowsocks Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавить сервер Shadowsocks...",
        "ZH": "添加Shadowsocks服务器...",
        "isReviewed": "True"
    },
    "Add Trojan Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавит сервер Trojan...",
        "ZH": "添加Trojan服务器...",
        "isReviewed": "True"
    },
    "Add Hysteria1 Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавьте сервер Hysteria 1...",
        "ZH": "添加Hysteria1服务器...",
        "isReviewed": "True"
    },
    "Add Hysteria2 Server...": {
        "source": [
            "Furious.Window.AppMainWindow"
        ],
        "RU": "Добавьте сервер Hysteria 2...",
        "ZH": "添加Hysteria2服务器...",
        "isReviewed": "True"
    },
    "Routing option with direct rules is not allowed in VPN mode": {
        "source": [
            "Furious.Core.CoreManager"
        ],
        "RU": "Опция маршрутизации с прямыми правилами не разрешена в режиме VPN",
        "ZH": "VPN模式下不允许使用直接规则的路由选项",
        "isReviewed": "True"
    },
    "Select and press Enter to activate configuration and connect": {
        "source": [
            "Furious.TrayActions.Connect"
        ],
        "RU": "Выберите и нажмите Enter, чтобы активировать конфигурацию и подключиться",
        "ZH": "选择并按下Enter键以激活配置并连接",
        "isReviewed": "True"
    }
}



# Copyright (C) 2024  Loren Eteval <loren.eteval@proton.me>
#
# This file is part of Furious.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from __future__ import annotations

from Furious.Interface import *
from Furious.QtFramework import *
from Furious.QtFramework import gettext as _
from Furious.Utility import *
from Furious.Widget.GuiVTLS import *

from PySide6.QtWidgets import *

from typing import Callable

__all__ = ['GuiVTransportQGroupBox']

needTrans = functools.partial(needTransFn, source=__name__)

STREAM_NETWORK = [
    'tcp',
    'kcp',
    'ws',
    'h2',
    'quic',
    'grpc',
    'httpupgrade',
]


class GuiVTransportItemNetwork(GuiEditorItemTextComboBox):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.addItems(STREAM_NETWORK)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        try:
            oldNetwork = streamSettings['network']
        except Exception:
            # Any non-exit exceptions

            oldNetwork = ''

        newNetwork = self.text()

        def setNewNetwork():
            streamSettings['network'] = newNetwork

            for network in STREAM_NETWORK:
                if network == newNetwork:
                    continue

                if network == 'h2':
                    networkKey = 'httpSettings'
                else:
                    networkKey = f'{network}Settings'

                # Remove irrelevant settings
                streamSettings.pop(networkKey, None)

        if isinstance(oldNetwork, str):
            if newNetwork != oldNetwork:
                setNewNetwork()

                return True
            else:
                return False
        else:
            setNewNetwork()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            streamSettings = getXrayProxyOutboundStream(config)

            self.setText(streamSettings.get('network', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemTypeXXX(GuiEditorItemTextComboBox):
    def __init__(self, *args, **kwargs):
        networkKey = kwargs.pop('networkKey', '')

        super().__init__(*args, **kwargs)

        self.networkKey = networkKey

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get(self.networkKey), dict):
            streamSettings[self.networkKey] = {}

        xxxObject = streamSettings[self.networkKey]

        try:
            oldType = xxxObject['header']['type']
        except Exception:
            # Any non-exit exceptions

            oldType = ''

        newType = self.text()

        def setNewType():
            if not isinstance(xxxObject.get('header'), dict):
                xxxObject['header'] = {}

            if newType == '':
                xxxObject['header'].pop('type', None)
            else:
                xxxObject['header']['type'] = newType

        if isinstance(oldType, str):
            if newType != oldType:
                setNewType()

                return True
            else:
                return False
        else:
            setNewType()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            xxxObject = getXrayProxyOutboundStream(config)[self.networkKey]

            self.setText(xxxObject['header']['type'])
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemTypeTcp(GuiVTransportItemTypeXXX):
    def __init__(self, *args, **kwargs):
        networkKey = kwargs.pop('networkKey', 'tcpSettings')

        super().__init__(*args, **kwargs, networkKey=networkKey)

        self.addItems(
            [
                '',
                'none',
                'http',
            ]
        )


class GuiVTransportItemHostTcp(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('tcpSettings'), dict):
            streamSettings['tcpSettings'] = {}

        tcpObject = streamSettings['tcpSettings']

        try:
            oldHost = ','.join(tcpObject['header']['request']['headers']['Host'])
        except Exception:
            # Any non-exit exceptions

            oldHost = ''

        newHost = self.text()

        def setNewHost():
            if not isinstance(tcpObject.get('header'), dict):
                tcpObject['header'] = {}

            if not isinstance(tcpObject['header'].get('request'), dict):
                tcpObject['header']['request'] = {}

            if not isinstance(tcpObject['header']['request'].get('headers'), dict):
                tcpObject['header']['request']['headers'] = {}

            if newHost == '':
                tcpObject['header']['request']['headers'].pop('Host', None)
            else:
                tcpObject['header']['request']['headers']['Host'] = newHost.split(',')

        if isinstance(oldHost, str):
            if newHost != oldHost:
                setNewHost()

                return True
            else:
                return False
        else:
            setNewHost()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            tcpObject = getXrayProxyOutboundStream(config)['tcpSettings']

            self.setText(','.join(tcpObject['header']['request']['headers']['Host']))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemPathTcp(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('tcpSettings'), dict):
            streamSettings['tcpSettings'] = {}

        tcpObject = streamSettings['tcpSettings']

        try:
            oldPath = ','.join(tcpObject['header']['request']['path'])
        except Exception:
            # Any non-exit exceptions

            oldPath = ''

        newPath = self.text()

        def setNewPath():
            if not isinstance(tcpObject.get('header'), dict):
                tcpObject['header'] = {}

            if not isinstance(tcpObject['header'].get('request'), dict):
                tcpObject['header']['request'] = {}

            if newPath == '':
                tcpObject['header']['request'].pop('path', None)
            else:
                tcpObject['header']['request']['path'] = newPath.split(',')

        if isinstance(oldPath, str):
            if newPath != oldPath:
                setNewPath()

                return True
            else:
                return False
        else:
            setNewPath()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            tcpObject = getXrayProxyOutboundStream(config)['tcpSettings']

            self.setText(','.join(tcpObject['header']['request']['path']))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemTypeKcp(GuiVTransportItemTypeXXX):
    def __init__(self, *args, **kwargs):
        networkKey = kwargs.pop('networkKey', 'kcpSettings')

        super().__init__(*args, **kwargs, networkKey=networkKey)

        self.addItems(
            [
                '',
                'none',
                'srtp',
                'utp',
                'wechat-video',
                'dtls',
                'wireguard',
                'dns',
            ]
        )


class GuiVTransportItemSeedKcp(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('kcpSettings'), dict):
            streamSettings['kcpSettings'] = {}

        kcpObject = streamSettings['kcpSettings']

        try:
            oldSeed = kcpObject.get('seed', '')
        except Exception:
            # Any non-exit exceptions

            oldSeed = ''

        newSeed = self.text()

        def setNewSeed():
            if newSeed == '':
                kcpObject.pop('seed', None)
            else:
                kcpObject['seed'] = newSeed

        if isinstance(oldSeed, str):
            if newSeed != oldSeed:
                setNewSeed()

                return True
            else:
                return False
        else:
            setNewSeed()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            kcpObject = getXrayProxyOutboundStream(config)['kcpSettings']

            self.setText(kcpObject.get('seed', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemHostWs(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('wsSettings'), dict):
            streamSettings['wsSettings'] = {}

        wsObject = streamSettings['wsSettings']

        try:
            oldHost = wsObject['headers']['Host']
        except Exception:
            # Any non-exit exceptions

            oldHost = ''

        newHost = self.text()

        def setNewHost():
            if not isinstance(wsObject.get('headers'), dict):
                wsObject['headers'] = {}

            if newHost == '':
                wsObject['headers'].pop('Host', None)
            else:
                wsObject['headers']['Host'] = newHost

        if isinstance(oldHost, str):
            if newHost != oldHost:
                setNewHost()

                return True
            else:
                return False
        else:
            setNewHost()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            wsObject = getXrayProxyOutboundStream(config)['wsSettings']

            self.setText(wsObject['headers']['Host'])
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemPathWs(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('wsSettings'), dict):
            streamSettings['wsSettings'] = {}

        wsObject = streamSettings['wsSettings']

        try:
            oldPath = wsObject.get('path', '')
        except Exception:
            # Any non-exit exceptions

            oldPath = ''

        newPath = self.text()

        def setNewPath():
            if newPath == '':
                wsObject.pop('path', None)
            else:
                wsObject['path'] = newPath

        if isinstance(oldPath, str):
            if newPath != oldPath:
                setNewPath()

                return True
            else:
                return False
        else:
            setNewPath()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            wsObject = getXrayProxyOutboundStream(config)['wsSettings']

            self.setText(wsObject.get('path', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemHostHttpUpgrade(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('httpupgradeSettings'), dict):
            streamSettings['httpupgradeSettings'] = {}

        httpUpgradeObject = streamSettings['httpupgradeSettings']

        try:
            oldHost = httpUpgradeObject.get('host', '')
        except Exception:
            # Any non-exit exceptions

            oldHost = ''

        newHost = self.text()

        def setNewHost():
            if newHost == '':
                httpUpgradeObject.pop('host', None)
            else:
                httpUpgradeObject['host'] = newHost

        if isinstance(oldHost, str):
            if newHost != oldHost:
                setNewHost()

                return True
            else:
                return False
        else:
            setNewHost()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            httpUpgradeObject = getXrayProxyOutboundStream(config)[
                'httpupgradeSettings'
            ]

            self.setText(httpUpgradeObject.get('host', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemPathHttpUpgrade(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('httpupgradeSettings'), dict):
            streamSettings['httpupgradeSettings'] = {}

        httpUpgradeObject = streamSettings['httpupgradeSettings']

        try:
            oldPath = httpUpgradeObject.get('path', '')
        except Exception:
            # Any non-exit exceptions

            oldPath = ''

        newPath = self.text()

        def setNewPath():
            if newPath == '':
                httpUpgradeObject.pop('path', None)
            else:
                httpUpgradeObject['path'] = newPath

        if isinstance(oldPath, str):
            if newPath != oldPath:
                setNewPath()

                return True
            else:
                return False
        else:
            setNewPath()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            httpUpgradeObject = getXrayProxyOutboundStream(config)[
                'httpupgradeSettings'
            ]

            self.setText(httpUpgradeObject.get('path', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemHostH2(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('httpSettings'), dict):
            streamSettings['httpSettings'] = {}

        httpObject = streamSettings['httpSettings']

        try:
            oldHost = ','.join(httpObject.get('host', []))
        except Exception:
            # Any non-exit exceptions

            oldHost = ''

        newHost = self.text()

        def setNewHost():
            if newHost == '':
                httpObject.pop('host', None)
            else:
                httpObject['host'] = newHost.split(',')

        if isinstance(oldHost, str):
            if newHost != oldHost:
                setNewHost()

                return True
            else:
                return False
        else:
            setNewHost()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            httpObject = getXrayProxyOutboundStream(config)['httpSettings']

            self.setText(','.join(httpObject.get('host', [])))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemPathH2(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('httpSettings'), dict):
            streamSettings['httpSettings'] = {}

        httpObject = streamSettings['httpSettings']

        try:
            oldPath = httpObject.get('path', '')
        except Exception:
            # Any non-exit exceptions

            oldPath = ''

        newPath = self.text()

        def setNewPath():
            if newPath == '':
                httpObject.pop('path', None)
            else:
                httpObject['path'] = newPath

        if isinstance(oldPath, str):
            if newPath != oldPath:
                setNewPath()

                return True
            else:
                return False
        else:
            setNewPath()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            httpObject = getXrayProxyOutboundStream(config)['httpSettings']

            self.setText(httpObject.get('path', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemTypeQuic(GuiVTransportItemTypeXXX):
    def __init__(self, *args, **kwargs):
        networkKey = kwargs.pop('networkKey', 'quicSettings')

        super().__init__(*args, **kwargs, networkKey=networkKey)

        self.addItems(
            [
                '',
                'none',
                'srtp',
                'utp',
                'wechat-video',
                'dtls',
                'wireguard',
            ]
        )


class GuiVTransportItemSecurityQuic(GuiEditorItemTextComboBox):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.addItems(
            [
                '',
                'none',
                'aes-128-gcm',
                'chacha20-poly1305',
            ]
        )

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('quicSettings'), dict):
            streamSettings['quicSettings'] = {}

        quicObject = streamSettings['quicSettings']

        try:
            oldSecurity = quicObject.get('security', '')
        except Exception:
            # Any non-exit exceptions

            oldSecurity = ''

        newSecurity = self.text()

        def setNewSecurity():
            quicObject['security'] = newSecurity

        if isinstance(oldSecurity, str):
            if newSecurity != oldSecurity:
                setNewSecurity()

                return True
            else:
                return False
        else:
            setNewSecurity()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            quicObject = getXrayProxyOutboundStream(config)['quicSettings']

            self.setText(quicObject.get('security', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemKeyQuic(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('quicSettings'), dict):
            streamSettings['quicSettings'] = {}

        quicObject = streamSettings['quicSettings']

        try:
            oldKey = quicObject.get('key', '')
        except Exception:
            # Any non-exit exceptions

            oldKey = ''

        newKey = self.text()

        def setNewKey():
            if newKey == '':
                quicObject.pop('key', None)
            else:
                quicObject['key'] = newKey

        if isinstance(oldKey, str):
            if newKey != oldKey:
                setNewKey()

                return True
            else:
                return False
        else:
            setNewKey()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            quicObject = getXrayProxyOutboundStream(config)['quicSettings']

            self.setText(quicObject.get('key', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemModeGRPC(GuiEditorItemTextComboBox):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.addItems(
            [
                '',
                'gun',
                'multi',
            ]
        )

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('grpcSettings'), dict):
            streamSettings['grpcSettings'] = {}

        grpcObject = streamSettings['grpcSettings']

        try:
            oldMode = grpcObject.get('multiMode', False)
        except Exception:
            # Any non-exit exceptions

            oldMode = False

        if oldMode is True:
            oldMode = 'multi'
        else:
            oldMode = 'gun'

        newMode = self.text()

        def setNewMode():
            if newMode == 'multi':
                grpcObject['multiMode'] = True
            else:
                grpcObject['multiMode'] = False

        if isinstance(oldMode, str):
            if newMode != oldMode:
                setNewMode()

                return True
            else:
                return False
        else:
            setNewMode()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            grpcObject = getXrayProxyOutboundStream(config)['grpcSettings']

            multi = grpcObject.get('multiMode', False)

            if multi is True:
                self.setText('multi')
            elif multi is False:
                self.setText('gun')
            else:
                self.setText('')
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemAuthorityGRPC(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('grpcSettings'), dict):
            streamSettings['grpcSettings'] = {}

        grpcObject = streamSettings['grpcSettings']

        try:
            oldAuthority = grpcObject.get('authority', '')
        except Exception:
            # Any non-exit exceptions

            oldAuthority = ''

        newAuthority = self.text()

        def setNewAuthority():
            if newAuthority == '':
                grpcObject.pop('authority', None)
            else:
                grpcObject['authority'] = newAuthority

        if isinstance(oldAuthority, str):
            if newAuthority != oldAuthority:
                setNewAuthority()

                return True
            else:
                return False
        else:
            setNewAuthority()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            grpcObject = getXrayProxyOutboundStream(config)['grpcSettings']

            self.setText(grpcObject.get('authority', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportItemServiceNameGRPC(GuiEditorItemTextInput):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        streamSettings = getXrayProxyOutboundStream(config)

        if not isinstance(streamSettings.get('grpcSettings'), dict):
            streamSettings['grpcSettings'] = {}

        grpcObject = streamSettings['grpcSettings']

        try:
            oldServiceName = grpcObject.get('serviceName', '')
        except Exception:
            # Any non-exit exceptions

            oldServiceName = ''

        newServiceName = self.text()

        def setNewServiceName():
            if newServiceName == '':
                grpcObject.pop('serviceName', None)
            else:
                grpcObject['serviceName'] = newServiceName

        if isinstance(oldServiceName, str):
            if newServiceName != oldServiceName:
                setNewServiceName()

                return True
            else:
                return False
        else:
            setNewServiceName()

            return True

    def factoryToInput(self, config: ConfigurationFactory):
        try:
            grpcObject = getXrayProxyOutboundStream(config)['grpcSettings']

            self.setText(grpcObject.get('serviceName', ''))
        except Exception:
            # Any non-exit exceptions

            self.setText('')


class GuiVTransportPageXXX(GuiEditorWidgetQWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def setNetworkText(self, text: str):
        network = self._containers[0]

        if isinstance(network, GuiEditorItemTextComboBox):
            network.setText(text)

    def connectActivated(self, func: Callable):
        network = self._containers[0]

        if isinstance(network, GuiEditorItemTextComboBox):
            network.connectActivated(func)


class GuiVTransportPageTcp(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemTypeTcp(title='Type', translatable=False),
            GuiVTransportItemHostTcp(title='Host', translatable=False),
            GuiVTransportItemPathTcp(title='Path', translatable=False),
        ]


class GuiVTransportPageKcp(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemTypeKcp(title='Type', translatable=False),
            GuiVTransportItemSeedKcp(title='KCP seed', translatable=False),
        ]


class GuiVTransportPageWs(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemHostWs(title='Host', translatable=False),
            GuiVTransportItemPathWs(title='Path', translatable=False),
        ]


class GuiVTransportPageHttpUpgrade(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemHostHttpUpgrade(title='Host', translatable=False),
            GuiVTransportItemPathHttpUpgrade(title='Path', translatable=False),
        ]


class GuiVTransportPageH2(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemHostH2(title='Host', translatable=False),
            GuiVTransportItemPathH2(title='Path', translatable=False),
        ]


class GuiVTransportPageQuic(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemTypeQuic(title='Type', translatable=False),
            GuiVTransportItemSecurityQuic(title='QUIC Security', translatable=False),
            GuiVTransportItemKeyQuic(title='QUIC Key', translatable=False),
        ]


class GuiVTransportPageGRPC(GuiVTransportPageXXX):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def containerSequence(self):
        return [
            GuiVTransportItemNetwork(title='Network', translatable=False),
            GuiVTransportItemModeGRPC(title='gRPC mode', translatable=False),
            GuiVTransportItemAuthorityGRPC(title='gRPC authority', translatable=False),
            GuiVTransportItemServiceNameGRPC(
                title='gRPC serviceName', translatable=False
            ),
        ]


class GuiVTransportPageStackedWidget(QStackedWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Corresponds to stream network
        self._pages = [
            GuiVTransportPageTcp(),
            GuiVTransportPageKcp(),
            GuiVTransportPageWs(),
            GuiVTransportPageH2(),
            GuiVTransportPageQuic(),
            GuiVTransportPageGRPC(),
            GuiVTransportPageHttpUpgrade(),
        ]

        for page in self._pages:
            self.addWidget(page)

    def page(self, index: int) -> GuiVTransportPageXXX:
        return self._pages[index]

    def connectActivated(self, func: Callable):
        for page in self._pages:
            page.connectActivated(func)


needTrans(
    'Transport',
)


class GuiVTransportQGroupBox(GuiEditorItemFactory, AppQGroupBox):
    def __init__(self, **kwargs):
        super().__init__(_('Transport'), **kwargs)

        self._config = ConfigurationFactory()

        self._widget = GuiVTransportPageStackedWidget()
        self._widget.connectActivated(self.handleActivated)

        layout = QFormLayout()
        layout.addRow(self._widget)

        self.setLayout(layout)

    def currentIndex(self) -> int:
        return self._widget.currentIndex()

    def setCurrentIndex(self, index: int):
        self._widget.setCurrentIndex(index)

    def page(self, index: int) -> GuiVTransportPageXXX:
        return self._widget.page(index)

    def handleActivated(self, index: int):
        page = self.page(index)
        page.factoryToInput(self._config)
        page.setNetworkText(STREAM_NETWORK[index])

        self.setCurrentIndex(index)

    def inputToFactory(self, config: ConfigurationFactory) -> bool:
        return self.page(self.currentIndex()).inputToFactory(config)

    def factoryToInput(self, config: ConfigurationFactory):
        # Shallow copy
        self._config = config

        streamSettings = getXrayProxyOutboundStream(config)
        network = streamSettings.get('network', '')

        if not isinstance(network, str):
            return

        def alterNetwork(sourceNetwork, targetNetwork):
            nonlocal network

            if network == sourceNetwork:
                network = targetNetwork

            streamSettings['network'] = network

        # Adjust alternative values
        # https://xtls.github.io/config/transport.html#streamsettingsobject
        alterNetwork('http', 'h2')
        alterNetwork('gun', 'grpc')
        alterNetwork('mkcp', 'kcp')

        try:
            index = STREAM_NETWORK.index(network)
        except Exception:
            # Any non-exit exceptions

            pass
        else:
            self.page(index).factoryToInput(config)
            self.setCurrentIndex(index)



# Copyright (C) 2024  Loren Eteval <loren.eteval@proton.me>
#
# This file is part of Furious.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from __future__ import annotations

from Furious.Interface import *
from Furious.PyFramework import *
from Furious.QtFramework import *
from Furious.QtFramework import gettext as _
from Furious.Library import *
from Furious.Utility import *
from Furious.Core import *

import uuid
import logging
import functools
import subprocess

__all__ = ['CoreManager']

logger = logging.getLogger(__name__)

needTrans = functools.partial(needTransFn, source=__name__)

needTrans(
    'Unable to connect',
    'Routing option with direct rules is not allowed in VPN mode',
)


def fixLogObjectPath(config: ConfigurationFactory, attr: str, value: str, log=True):
    try:
        path = config['log'][attr]
    except Exception:
        # Any non-exit exceptions

        config['log'][attr] = path = ''

    if not isinstance(path, str) and not isinstance(path, bytes):
        config['log'][attr] = path = ''

    if path == '':
        if isPythonw() and StdoutRedirectHelper.TemporaryDir.isValid():
            # Redirect implementation for pythonw environment
            config['log'][attr] = StdoutRedirectHelper.TemporaryDir.filePath(value)
    else:
        # Relative path fails if booting on start up
        # on Windows, when packed using nuitka...

        # Fix relative path if needed. User cannot feel this operation.
        config['log'][attr] = getAbsolutePath(path)

    result = config['log'][attr]

    if result:
        try:
            # Create a new file
            with open(result, 'x'):
                pass
        except FileExistsError:
            pass
        except Exception:
            # Any non-exit exceptions

            pass

    if log:
        logger.info(
            f'{XrayCore.name()}: {attr} log is specified as \'{path}\'. '
            f'Fixed to \'{result}\''
        )


class CoreManager(SupportExitCleanup):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.coresPool = []

    def start(
        self,
        config: ConfigurationFactory,
        routing: str,
        exitCallback=None,
        msgCallback=None,
        tunMsgCallback=None,
        deepcopy=True,
        proxyModeOnly=False,
        log=True,
        **kwargs,
    ) -> bool:
        if deepcopy:
            copy = config.deepcopy()
        else:
            copy = config

        if isinstance(copy, ConfigurationXray):
            if copy.get('log') is None or not isinstance(copy['log'], dict):
                copy['log'] = {
                    'access': '',
                    'error': '',
                    'loglevel': 'warning',
                }

            logRedirectValue = str(uuid.uuid4())

            # Fix logObject
            for attr in ['access', 'error']:
                fixLogObjectPath(copy, attr, logRedirectValue, log)

            if routing == 'Bypass Mainland China':
                # VPN Mode handling
                if not proxyModeOnly and isVPNMode():
                    mbox = AppQMessageBox(icon=AppQMessageBox.Icon.Critical)
                    mbox.setWindowTitle(_('Unable to connect'))
                    mbox.setText(
                        _('Routing option with direct rules is not allowed in VPN mode')
                    )

                    # Show the MessageBox asynchronously
                    mbox.open()

                    return False

                routingObject = {
                    'domainStrategy': 'IPIfNonMatch',
                    'domainMatcher': 'hybrid',
                    'rules': [
                        {
                            'type': 'field',
                            'domain': [
                                'geosite:category-ads-all',
                            ],
                            'outboundTag': 'block',
                        },
                        {
                            'type': 'field',
                            'domain': [
                                'geosite:cn',
                            ],
                            'outboundTag': 'direct',
                        },
                        {
                            'type': 'field',
                            'ip': [
                                'geoip:private',
                                'geoip:cn',
                            ],
                            'outboundTag': 'direct',
                        },
                        {
                            'type': 'field',
                            'port': '0-65535',
                            'outboundTag': 'proxy',
                        },
                    ],
                }
            # elif routing == 'Bypass Iran':
            #     routingObject = {
            #         'domainStrategy': 'IPIfNonMatch',
            #         'domainMatcher': 'hybrid',
            #         'rules': [
            #             {
            #                 'type': 'field',
            #                 'domain': [
            #                     'geosite:category-ads-all',
            #                     'iran:ads',
            #                 ],
            #                 'outboundTag': 'block',
            #             },
            #             {
            #                 'type': 'field',
            #                 'domain': [
            #                     'iran:ir',
            #                     'iran:other',
            #                 ],
            #                 'outboundTag': 'direct',
            #             },
            #             {
            #                 'type': 'field',
            #                 'ip': [
            #                     'geoip:private',
            #                     'geoip:ir',
            #                 ],
            #                 'outboundTag': 'direct',
            #             },
            #             {
            #                 'type': 'field',
            #                 'port': '0-65535',
            #                 'outboundTag': 'proxy',
            #             },
            #         ],
            #     }
            elif routing == 'Global':
                routingObject = {}
            elif routing == 'Custom':
                routingObject = copy.get('routing', {})
            else:
                routingObject = {}

            if log:
                logger.info(f'core {XrayCore.name()} configured')
                logger.info(f'routing is {routing}')
                logger.info(f'RoutingObject: {routingObject}')

            copy['routing'] = routingObject

            core = XrayCore(exitCallback=exitCallback, msgCallback=msgCallback)
            success = core.start(copy, **kwargs)
        elif isinstance(copy, ConfigurationHysteria1):
            if routing == 'Bypass Mainland China':
                # VPN Mode handling
                if not proxyModeOnly and isVPNMode():
                    mbox = AppQMessageBox(icon=AppQMessageBox.Icon.Critical)
                    mbox.setWindowTitle(_('Unable to connect'))
                    mbox.setText(
                        _('Routing option with direct rules is not allowed in VPN mode')
                    )

                    # Show the MessageBox asynchronously
                    mbox.open()

                    return False

                routingObject = {
                    'rule': DATA_DIR / 'hysteria' / 'bypass-mainland-China.acl',
                    'mmdb': DATA_DIR / 'hysteria' / 'country.mmdb',
                }
            # elif routing == 'Bypass Iran':
            #     routingObject = {
            #         'rule': DATA_DIR / 'hysteria' / 'bypass-Iran.acl',
            #         'mmdb': DATA_DIR / 'hysteria' / 'country.mmdb',
            #     }
            elif routing == 'Global':
                routingObject = {
                    'rule': '',
                    'mmdb': '',
                }
            elif routing == 'Custom':
                routingObject = {
                    'rule': copy.get('acl', ''),
                    'mmdb': copy.get('mmdb', ''),
                }
            else:
                routingObject = {
                    'rule': '',
                    'mmdb': '',
                }

            if log:
                logger.info(f'core {Hysteria1.name()} configured')
                logger.info(f'routing is {routing}')
                logger.info(f'RoutingObject: {routingObject}')

            core = Hysteria1(exitCallback=exitCallback, msgCallback=msgCallback)
            success = core.start(
                copy,
                Hysteria1.rule(routingObject.get('rule', '')),
                Hysteria1.mmdb(routingObject.get('mmdb', '')),
                **kwargs,
            )
        elif isinstance(copy, ConfigurationHysteria2):
            if log:
                logger.info(f'core {Hysteria2.name()} configured')

            core = Hysteria2(exitCallback=exitCallback, msgCallback=msgCallback)
            success = core.start(copy, **kwargs)
        else:
            core = None
            success = False

        if core is not None:
            self.coresPool.append(core)

        if not success:
            logger.error(f'core {core.name()} start failed')

            return success

        # VPN Mode handling
        if not proxyModeOnly and isVPNMode():
            # Currently VPN Mode is only supported on Windows and macOS
            if PLATFORM == 'Windows' or PLATFORM == 'Darwin':
                if PLATFORM == 'Windows':
                    # cleanup first
                    SystemRoutingTable.delete(
                        '0.0.0.0', APPLICATION_TUN_GATEWAY_ADDRESS
                    )

                defaultGateway = SystemRoutingTable.getDefaultGateway()

                if PLATFORM == 'Darwin':
                    # Need this?
                    defaultGateway = list(
                        # Filter TUN Gateway
                        filter(
                            lambda x: x != APPLICATION_TUN_GATEWAY_ADDRESS,
                            defaultGateway,
                        )
                    )

                if len(defaultGateway) != 1:
                    logger.error(f'bad default gateway: {defaultGateway}')

                    return False

                if PLATFORM == 'Windows':
                    gateway, interfaceIP = defaultGateway[0]
                else:
                    gateway, interfaceIP = defaultGateway[0], None

                tun = Tun2socks(exitCallback=exitCallback, msgCallback=tunMsgCallback)
                self.coresPool.append(tun)

                if not tun.start(
                    APPLICATION_TUN_DEVICE_NAME,
                    APPLICATION_TUN_NETWORK_INTERFACE_NAME,
                    'info',
                    f'socks5://{copy.socksProxyEndpoint()}',
                    '',
                ):
                    return False

                address = copy.itemAddress

                if not isValidIPAddress(address):
                    error, resolved = DNSResolver.resolve(
                        address, *parseHostPort(copy.httpProxyEndpoint())
                    )

                    if error:
                        logger.error(f'DNS resolution failed: {address}')

                        SystemRoutingTable.Relations.clear()

                        return False
                    else:
                        for address in resolved:
                            SystemRoutingTable.Relations.append([address, gateway])
                else:
                    SystemRoutingTable.Relations.append([address, gateway])

                if PLATFORM == 'Windows':
                    foundDevice = False

                    for counter in range(0, 10000, 100):
                        try:
                            result = runExternalCommand(
                                'ipconfig',
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                check=True,
                            )
                        except Exception:
                            # Any non-exit exceptions

                            break
                        else:
                            stdout = result.stdout.decode('utf-8', 'replace')

                            if stdout.find(APPLICATION_TUN_DEVICE_NAME) >= 0:
                                foundDevice = True

                                logger.info(
                                    f'find TUN device \'{APPLICATION_TUN_DEVICE_NAME}\' success. '
                                    f'Counter: {counter}'
                                )

                                break

                        PySide6LegacyEventLoopWait(100)

                    if not foundDevice:
                        logger.error(
                            f'find TUN device \'{APPLICATION_TUN_DEVICE_NAME}\' failed'
                        )

                        return False

                    alias = SystemRoutingTable.WIN32GetInterfaceAliasByIP(interfaceIP)

                    if alias:

                        def _windowsCleanup(_alias):
                            SystemRoutingTable.WIN32SetInterfaceDNS(_alias)
                            SystemRoutingTable.WIN32FlushDNSCache()

                        tun.cleanup = functools.partial(_windowsCleanup, alias)

                        SystemRoutingTable.WIN32SetInterfaceDNS(
                            alias, '127.0.0.1', False
                        )

                    SystemRoutingTable.addRelations()
                    SystemRoutingTable.WIN32SetInterfaceDNS(
                        APPLICATION_TUN_DEVICE_NAME,
                        APPLICATION_TUN_INTERFACE_DNS_ADDRESS,
                        False,
                    )
                    SystemRoutingTable.setDeviceGateway(
                        APPLICATION_TUN_DEVICE_NAME,
                        APPLICATION_TUN_IP_ADDRESS,
                        APPLICATION_TUN_GATEWAY_ADDRESS,
                    )
                    SystemRoutingTable.WIN32FlushDNSCache()

                if PLATFORM == 'Darwin':
                    for source in [
                        *list(f'{2 ** (8 - x)}.0.0.0/{x}' for x in range(8, 0, -1)),
                        '198.18.0.0/15',
                    ]:
                        SystemRoutingTable.Relations.append(
                            [source, APPLICATION_TUN_GATEWAY_ADDRESS]
                        )

                    servers = SystemRoutingTable.DarwinGetDNSServers()

                    def _darwinCleanup(_servers):
                        for _service, _dnsserver in _servers:
                            SystemRoutingTable.DarwinSetDNSServers(_service, _dnsserver)

                    tun.cleanup = functools.partial(_darwinCleanup, servers)

                    for service, dnsserver in servers:
                        SystemRoutingTable.DarwinSetDNSServers(
                            service,
                            APPLICATION_TUN_INTERFACE_DNS_ADDRESS,
                        )

                    SystemRoutingTable.setDeviceGateway(
                        APPLICATION_TUN_DEVICE_NAME,
                        APPLICATION_TUN_IP_ADDRESS,
                        APPLICATION_TUN_GATEWAY_ADDRESS,
                    )
                    SystemRoutingTable.addRelations()

        return True

    def allRunning(self) -> bool:
        return all(core.isRunning() for core in self.coresPool)

    def anyRunning(self) -> bool:
        return any(core.isRunning() for core in self.coresPool)

    def stopAll(self):
        if self.coresPool:
            for core in self.coresPool:
                if isinstance(core, CoreFactory):
                    core.stop()

            self.coresPool.clear()

    def cleanup(self):
        self.stopAll()
